/*******************************************************************************
* @file     hv.h
* @brief    High voltage module header file.
 * @version  V1.8.0
 * @date     31-Dec-2021
 *
 * @note
 * Copyright (C) 2021 Spintrol Limited. All rights reserved.
*
* @attention
* THIS SOFTWARE JUST PROVIDES CUSTOMERS WITH CODING INFORMATION REGARDING
* THEIR PRODUCTS, WHICH AIMS AT SAVING TIME FOR THEM. SPINTROL SHALL NOT BE
* LIABLE FOR THE USE OF THE SOFTWARE. SPINTROL DOES NOT GUARANTEE THE
* CORRECTNESS OF THIS SOFTWARE AND RESERVES THE RIGHT TO MODIFY THE SOFTWARE
* WITHOUT NOTIFICATION.
*
******************************************************************************/

#ifndef HV_H
#define HV_H

#ifdef __cplusplus
extern "C" {
#endif


#include "epwr.h"
#include "hv_bitfield.h"




/**
 *  @brief  High Voltage Module Register Address Definition
 */
typedef enum
{
    HV_REG_GLBSTS0                  = 0x00,   /*!< Global Status Register 0                 */
    HV_REG_GLBSTS1                  = 0x01,   /*!< Global Status Register 1                 */
    HV_REG_GLBSTSCLR0               = 0x02,   /*!< Global Status Clear Register 0           */
    HV_REG_GLBSTSCLR1               = 0x03,   /*!< Global Status Clear Register 1           */
    HV_REG_PDRVTZ0SEL0              = 0x04,   /*!< PDRVTZ0 Select Register 0                */
    HV_REG_PDRVTZ0SEL1              = 0x05,   /*!< PDRVTZ0 Select Register 1                */
    HV_REG_PDRVTZ1SEL0              = 0x06,   /*!< PDRVTZ1 Select Register 0                */
    HV_REG_PDRVTZ1SEL1              = 0x07,   /*!< PDRVTZ1 Select Register 1                */
    HV_REG_BODVBATVTH               = 0x08,   /*!< VBAT BOD Threshold Register              */
    HV_REG_BODVDDGVTH               = 0x09,   /*!< VDDG BOD Threshold Register              */
    HV_REG_OTTH                     = 0x0A,   /*!< Over-Temperature Threshold Register      */
    HV_REG_SYSCTL                   = 0x0B,   /*!< System Control Register                  */
    HV_REG_BUCKCTL                  = 0x0C,   /*!< BUCK Control Register                    */
    HV_REG_SSCTL                    = 0x0D,   /*!< Spread-Spectrum Control Register         */
    HV_REG_PDRVCTL                  = 0x0E,   /*!< Pre-Driver Control Register              */
    HV_REG_PDRVOCCTL                = 0x0F,   /*!< Pre-Driver Over-Current Control Register */
    HV_REG_VMONCTL                  = 0x10,   /*!< Supply Voltage Monitor Control Register  */
    HV_REG_CTLKEY                   = 0x11,   /*!< Control Key Register                     */

    /* Registers For Engineering Test */
    HV_REG_OSCTRIM                  = 0x12,   /*!< OSC Trim Register                        */
    HV_REG_LDORCOTRIM               = 0x13,   /*!< LDO and RCO Trim Register                */
    HV_REG_BODTRIM                  = 0x14,   /*!< BOD Reference Trim Register              */
    HV_REG_BUCKOCTRIM               = 0x15,   /*!< BUCK Over-Current Trim Register          */
    HV_REG_BUCKREFTRIM              = 0x16,   /*!< BUCK Reference Trim Register             */
    HV_REG_BUCKTONTRIM              = 0x17,   /*!< BUCK On-Time Trim Register               */
    HV_REG_BODENGR                  = 0x18,   /*!< BOD Engineering Register                 */
    HV_REG_MISCENGR                 = 0x19,   /*!< Misc Engineering Register                */
    HV_REG_BUCKENGR0                = 0x1A,   /*!< BUCK Engineering Register 0              */
    HV_REG_BUCKENGR1                = 0x1B,   /*!< BUCK Engineering Register 1              */
    HV_REG_CHPENGR                  = 0x1C,   /*!< Charge-Pump Enginnering Register         */
    HV_REG_ATESTEN                  = 0x1E,   /*!< ATEST Enable Register                    */
    HV_REG_TESTSEL                  = 0x1F,   /*!< DTEST Select Register                    */
} HV_REGS;




/**
 *  @brief  High Voltage Module Chage-pump clock dividing value definition
 */
typedef enum
{
    HV_CHARGEPUMP_CLKDIV_8          = 0x00,    /*!< Divide by 8  */
    HV_CHARGEPUMP_CLKDIV_16         = 0x01,    /*!< Divide by 16 */
    HV_CHARGEPUMP_CLKDIV_32         = 0x02,    /*!< Divide by 32 */
    HV_CHARGEPUMP_CLKDIV_64         = 0x03,    /*!< Divide by 64 */
} HV_ChargePumpClockDivEnum;




/**
 *  @brief  High Voltage Module Pin Interface Definition
 */
#define GPIO_HV_PWM_W_L           GPIO_24   /*!< GPIO for PWM_W_L           */
#define GPIO_HV_PWM_W_H           GPIO_25   /*!< GPIO for PWM_W_H           */
#define GPIO_HV_PWM_V_L           GPIO_26   /*!< GPIO for PWM_V_L           */
#define GPIO_HV_PWM_V_H           GPIO_27   /*!< GPIO for PWM_V_H           */
#define GPIO_HV_PWM_U_L           GPIO_28   /*!< GPIO for PWM_U_L           */
#define GPIO_HV_PWM_U_H           GPIO_29   /*!< GPIO for PWM_U_H           */




/**
 *  @brief  High Voltage Module Communication Command
 */
#define HV_CMD_READ_ID            (0x514E4944UL)
#define HV_CMD_WRITE              (0x5740005AUL)
#define HV_CMD_EXECUTE            (0x45786563UL)
#define HV_CMD_READ               (0x5240445AUL)
#define HV_CMD_RESET              (0x5253544EUL)
#define HV_CMD_DEEP_SLEEP         (0x5057444EUL)




/**
 *  @brief  High Voltage Module Masked Response
 */
#define HV_RESP_READ_ID           (0x49444B00UL)
#define HV_RESP_WRITE             (0x57A0005AUL)
#define HV_RESP_EXECUTE           (0x446F6E65UL)
#define HV_RESP_READ              (0x52A00000UL)
#define HV_RESP_RESET             (0x446F6E65UL)
#define HV_RESP_DEEP_SLEEP        (0x446F6E65UL)




/**
 *  @brief  High Voltage Module timeout count number
 */
#define HV_TIMEOUT_MAX            (100000U)




/**
 *  @brief  High Voltage Module SPI Communication Maximum Clock Frequency
 */
#define HV_SPI_MAX_CLK            (40000000)  /*!< 40 MHz */




/**
 *  @brief  High Voltage Module Register Address Check
 */
#define IS_HV_REG(u8Addr)         (((u8Addr) <= 0x1F) && ((u8Addr) != 0x1D))




/**
 *  @brief  High Voltage Module Public Function Declaration
 */
ErrorStatus HV_ComInit(uint32_t u32Baudrate);
ErrorStatus HV_Init(uint32_t u32Baudrate, uint8_t *pu8ID, uint32_t u32Timeout);

ErrorStatus HV_Handshake(uint8_t *pu8ID, uint32_t u32Timeout);
ErrorStatus HV_Command(uint32_t u32Req, uint32_t *pu32Resp);

ErrorStatus HV_WriteReg(uint8_t u8Addr, uint8_t u8Data);
ErrorStatus HV_ReadReg(uint8_t u8Addr, uint8_t *pu8Data);
ErrorStatus HV_WriteRegField(uint8_t u8Addr, uint8_t u8Mask, uint8_t u8FieldData);

ErrorStatus HV_InitReg(void);
ErrorStatus HV_ResetReg(void);
ErrorStatus HV_DeepSleep(void);

void HV_ConnectAnalogTestToADC(void);
ErrorStatus HV_ConnectTSensorToAnalogTest(void);

ErrorStatus HV_SetChargePumpClockDiv(HV_ChargePumpClockDivEnum eClockDiv);
ErrorStatus HV_GetChargePumpClockDiv(uint8_t *pu8Data);
ErrorStatus HV_EnableLowVoltageOperation(void);


#ifdef __cplusplus
}
#endif /* extern "C" */

#endif /* HV_H */


/******************* (C) COPYRIGHT 2021 SPINTROL LIMITED ***** END OF FILE ****/
