/******************************************************************************
 * @file     main.c
 * @brief    Main program body
 * @version  V1.8.0
 * @date     31-Dec-2021
 *
 * @note
 * Copyright (C) 2021 Spintrol Limited Corporation. All rights reserved.
 *
 * @attention
 * THIS SOFTWARE JUST PROVIDES CUSTOMERS WITH CODING INFORMATION REGARDING
 * THEIR PRODUCTS, WHICH AIMS AT SAVING TIME FOR THEM. SPINTROL SHALL NOT BE
 * LIABLE FOR THE USE OF THE SOFTWARE. SPINTROL DOES NOT GUARANTEE THE
 * CORRECTNESS OF THIS SOFTWARE AND RESERVES THE RIGHT TO MODIFY THE SOFTWARE
 * WITHOUT NOTIFICATION.
 *
 ******************************************************************************/

#include"spc1168.h"
#include<stdio.h>

/*As descripted below, ADC result convert to voltage can calculate as the follow*/
#define               ValueToVoltage(x)            ((x * 3657) / 8192) /1000
#define               SAMPLETIMENS                 300
#define               CONVTIMENS                   300

int16_t               i16VSP                       = -1;


/*************************************************************************************************************************
 *
 * @brief      In this case, a voltage was given to the GPIO2/0/9, then ADC Sampler A/B/C will sample the voltage
 *             with SOC2/SOC0/SOC9 respectively, after SOC2 had converted done, the EOC will trigger SOC9 start to work
 *             (SOC9 is the next SOC in round robin normally), when SOC9 convertion is done, the EOC will trigger SOC0. 
 *             As a result, SOC2, SOC9, SOC0 will work one by one.
 *
 *************************************************************************************************************************/
 

int main()
{
  FLASH_WALLOW();
  FLASH_SetTiming(200000000);
  /* Disable flash write access after flash operation had done */
  FLASH_WDIS();

  CLOCK_InitWithRCO(CLOCK_HCLK_200MHZ);

  Delay_Init();

  /*
   * Init the UART
   *
   * 1.Set the GPIO34/35 as UART FUNC
   *
   * 2.Enable the UART CLK
   *
   * 3.Set the rest para
   */
  GPIO_SetPinChannel(GPIO_34,GPIO34_UART_TXD);
  GPIO_SetPinChannel(GPIO_35,GPIO35_UART_RXD);
  CLOCK_EnableModule(UART_MODULE);
  UART_Init(UART,38400);

  /* 
   * ADC Init, Set GPIO2/0 as ADC Sampler A/B Positive pin respectively, the Negative pin of them is GND. GPIO9 is ADC 
   * Sampler C Negative pin, and the Positive pin is GND.
   */
  ADC_EasyInit1(ADC_SOC_2,ADC2_GPIO2,ADCTRIG_Software);
  ADC_EasyInit2(ADC_SOC_0,         0,ADC0_GPIO0,ADCTRIG_Software);
  ADC_EasyInit2(ADC_SOC_9,         0,ADC9_GPIO9,ADCTRIG_Software);
  
  /* 
   * Default convert and sample time was 150ns in 'ADC_EasyInit1', reset the value here .
   * if the value is too small, when we handle event in SOC2 INT Handler, SOC0 and SOC9 may be done during the SOC2 handler period,
   * as a result, when SOC2 INT done, SOC0 and SOC9 INT come to CPU simutaneously, then CPU will process SOC0 INT firstly, and then
   * SOC9 INT, this is not we want.
   */
  ADC_SetSampleAndConvertTime(ADC_SOC_2,SAMPLETIMENS,CONVTIMENS);
  ADC_SetSampleAndConvertTime(ADC_SOC_0,SAMPLETIMENS,CONVTIMENS);
  ADC_SetSampleAndConvertTime(ADC_SOC_9,SAMPLETIMENS,CONVTIMENS);

  /* Enbale SOC0/9 can be triggered by signal come from ADC EOC(end of convert) INT */
  ADC_EnableIntTriggerSOC(ADC_SOC_0);
  ADC_EnableIntTriggerSOC(ADC_SOC_9);

  /* Set the trigger source of SOC0/9 as the EOC of SOC2 */
  ADC_SelectIntTriggerSOC(ADC_SOC_0, ADC_INT_2);
  ADC_SelectIntTriggerSOC(ADC_SOC_9, ADC_INT_2);

  /* Enable G-INT for SOC2/0/9 */
  NVIC_EnableIRQ(ADC2_IRQn);
  NVIC_EnableIRQ(ADC0_IRQn);
  NVIC_EnableIRQ(ADC9_IRQn);

  /* Use software to trigger SOC2 to work */
  ADC_SoftwareTrigger(ADC_SOC_2);

  while(1)
  {

  }
}

void ADC2_IRQHandler(void)
{
  /* Result gotten from 'ADC_GetResult()' can be a negtive value or a positive value */ 
  i16VSP = ADC_GetResult(ADC_SOC_2);

  /* Result gotten from 'ADC_GetTrimResult1()' is a positive value */
  i16VSP = ADC_GetTrimResult1(ADC_SOC_2);

  /* Clear SOC INT */
  ADC_ClearInt2();

  /* Clear SOC INT flag */
  ADC_ClearInt(ADC_SOC_2);

  i16VSP = -1;
}

void ADC0_IRQHandler(void)
{
  /* Result gotten from 'ADC_GetResult()' can be a negtive value or a positive value */
  i16VSP = ADC_GetResult(ADC_SOC_0);

  printf("ADC SOC0 Voltage = %2fV(data is %d)\n", (double)ValueToVoltage(i16VSP),i16VSP);

  /* Result gotten from 'ADC_GetTrimResult1()' is a positive value */
  i16VSP = ADC_GetTrimResult1(ADC_SOC_0);

  printf("ADC SOC0 Voltage = %2fV(Trim data is %d)\n", (double)ValueToVoltage(i16VSP),i16VSP);

  /* Clear SOC INT */
  ADC_ClearInt0();

  /* Clear SOC INT flag */
  ADC_ClearInt(ADC_SOC_0);

  i16VSP = -1;
}

void ADC9_IRQHandler(void)
{
  /* Result gotten from 'ADC_GetResult()' can be a negtive value or a positive value */
  i16VSP = ADC_GetResult(ADC_SOC_9);

  printf("ADC SOC9 Voltage = %2fV(data is %d)\n", (double)ValueToVoltage(i16VSP),i16VSP);

  /* Result gotten from 'ADC_GetTrimResult1()' is a positive value */
  i16VSP = ADC_GetTrimResult1(ADC_SOC_9);

  printf("ADC SOC9 Voltage = %2fV(trim data is %d)\n", (double)ValueToVoltage(i16VSP),i16VSP);

  /* Clear soc1 INT */
  ADC_ClearInt9();

  /* Clear soc1 INT flag */
  ADC_ClearInt(ADC_SOC_9);

  i16VSP = -1;
}
