/**************************************************************************//**
 * @file     main.c
 * @brief    Main program body
 * @version  V1.1.0
 * @date     14-Feb-2019
 *
 * @note
 * Copyright (C) 2019 Spintrol Limited Corporation. All rights reserved.
 *
 * @attention
 * THIS SOFTWARE JUST PROVIDES CUSTOMERS WITH CODING INFORMATION REGARDING 
 * THEIR PRODUCTS, WHICH AIMS AT SAVING TIME FOR THEM. SPINTROL SHALL NOT BE
 * LIABLE FOR THE USE OF THE SOFTWARE. SPINTROL DOES NOT GUARANTEE THE 
 * CORRECTNESS OF THIS SOFTWARE AND RESERVES THE RIGHT TO MODIFY THE SOFTWARE 
 * WITHOUT NOTIFICATION.
 *
 ******************************************************************************/

#include "spd1148.h"
#include <stdio.h>


/* This macro is used to get the PWM period with a specified frequency */
#define                     PWMPeriod(u32PWMFreqHz)           ((CLOCK_GetModuleClock(PWM_MODULE))/u32PWMFreqHz)/2;

#define                     PRE_DRIVER_PWM_U                             PWM5
#define                     PRE_DRIVER_PWM_V                             PWM4
#define                     PRE_DRIVER_PWM_W                             PWM3
#define                     PRE_DRIVER_LINK_PWM_U                        SEL_PWM5
#define                     PRE_DRIVER_PWM_FREQ                          10000                  /* 10kHz PWM */
#define                     PWM_DB_NS                                    1000                  /* 1us */

uint32_t                    u32PWMPeriod;
uint8_t                     u8PREDRIID                                    = 0;                 /* PRE-DRIVER mode ID */
uint8_t                     u8PREDRIDATA;                                                       /* Data read from PRE-DRIVER */
ErrorStatus                 eErrorState;
uint32_t                    u32SSPBaundrate                               = 1000000;           /* The baundrate of SSP communicating with PRE-DRIVER mode */
uint32_t                    u32Timeout                                    = 500;               /* The number of times try to get PRE-DRIVER mode ID */
uint8_t                     Count;


 /*************************************************************************************************************************
 *
 * @brief      This case show how to make pre-driver keep working with low-voltage (pre-driver remain functional with VBAT
 *             down to 5V).
 *
 *                                                          DVDD(3.3V by default)
 *                                                          / \
 *                                                           |       SPD1148  Chip
 *                                 --------------------------|----------------------------
 *                                |    High Voltage Part     |                            |
 *                                |    -----------------     |                            |
 *                                |   |      -------    |    |                            |
 *                         VBAT---|---|---->|       |---|----                             |
 *                                |   |     | BUCK  |   |   |                             |
 *                                |   |      -------    |   |                             |
 *                                |   |       |         |   |        Low voltage part     |
 *                                |   |       |         |   |      --------------------   |
 *                                |   |       |         |   |     |                    |  |
 *                                |   |      -------    |   |VCAP(1.2V)                |  |
 *                                |   |     |       |   |    ---->|         MCU        |  |
 *                                |   |     | LDO   |---|----     |                    |  |
 *                                |   |     |       |   |    |    |                    |  |
 *                                |   |      -------    |    |    |                    |  |
 *                                |    -----------------     |     --------------------   |
 *                                 --------------------------|----------------------------
 *                                                           |
 *                                                          \ /
 *                                                          VDDG(10V by default)
 *
 *             Key point:
 *             PRE-DRIVER init include two step,
 *                                             <1> Init the communication circuits(SSP circyits) with MCU, we call this step
 *                                                 'Init PRE-DRIVER module'.
 *                                             <2> Init PRE-DRIVER socket itself, we call this step 'Power up PRE-DRIVER mode'.
 *
 *************************************************************************************************************************/


int main(void)
{
  FLASH_WALLOW();
  FLASH_SetTiming(200000000);
  /* Disable flash write access after flash operation had done */
  FLASH_WDIS();

  CLOCK_InitWithRCO(CLOCK_HCLK_200MHZ);

  Delay_Init();

  /*
   * Init the UART
   *
   * 1.Set the GPIO34/35 as UART FUNC
   *
   * 2.Enable the UART CLK
   *
   * 3.Set the rest para
   */
  GPIO_SetPinChannel(GPIO_34,GPIO34_UART_TXD);
  GPIO_SetPinChannel(GPIO_35,GPIO35_UART_RXD);
  CLOCK_EnableModule(UART_MODULE);
  UART_Init(UART,38400);
  
  /* Init PRE-DRIVER module(this step means 'Init PRE-DRIVER module') and get the ID */
  eErrorState = HV_Init(u32SSPBaundrate, &u8PREDRIID, u32Timeout);
  if(eErrorState == ERROR)
  {
    printf("Init PRE-DRIVER mode FAIL\n");
    return 0;
  }
  else
  {
    printf("Init PRE-DRIVER mode SUCCESS[ID:%d]\n",u8PREDRIID);
  }

  /* The step about enable low-voltage pre-driver operation, please refer to manual */
  /* Before operate PDRVCTL register, we need to set CTLKEY register to 0x99 */
  eErrorState = HV_WriteReg(HV_REG_CTLKEY, CTLKEY_ALL_KEY_PDRVEN);
  if(eErrorState == ERROR)
  {
    printf("Write CTLKEY register FAIL\n");
    return 0;
  }

  /* Power up PRE-DRIVER mode(this step means 'Power up PRE-DRIVER mode') */
  eErrorState = HV_WriteReg(HV_REG_PDRVCTL, PDRVCTL_ALL_EN_ENABLE);
  if(eErrorState == ERROR)
  {
    printf("Power up PRE-DRIVER FAIL\n");
    return 0;
  }
  
  /* Delay 100us to wait for the VDDG LDO ready */
  Delay_Us(100);
  
  /* Wait for PRE-DRIVER mode become enable */
  while(((u8PREDRIDATA & (0x1 << 6)) == 0) && (Count < 10))
  {
    eErrorState = HV_ReadReg(HV_REG_GLBSTS0, &u8PREDRIDATA);
    if(eErrorState == ERROR)
    {
      printf("PRE-DRIVER enable FAIL[%02x]\n",u8PREDRIDATA);
      return 0;
    }
    if(Count > 10)
    {
      printf("PRE-DRIVER enable timeout\n");
      return 0;
    }
    Count ++;
  }
  
  /* additional delay after VDDG ready, for final power settling */
  Delay_Us(100);

	/* Set 'SYSCTL.VDDGRDYBYP' */
  /* 1. Read the default value of SYSCTL register first */
  eErrorState = HV_ReadReg(HV_REG_SYSCTL, &u8PREDRIDATA);
  if(eErrorState == ERROR)
  {
    printf("Read SYSCTL FAIL\n");
    return 0;
  }
  /*2. Then set 'SYSCTL.VDDGRDYBYP'*/
  eErrorState = HV_WriteReg(HV_REG_SYSCTL, (u8PREDRIDATA & (~0x40)) | SYSCTL_ALL_VDDGRDYBYP_BYPASS);
  if(eErrorState == ERROR)
  {
    printf("Reset SYSCTL.VBATDIVEN bit FAIL\n");
    return ERROR;
  }
	
	/* Set 'PDRVCTL.FRCCFM' */
	/* 1. Read the default value of PDRVCTL register first */
  eErrorState = HV_ReadReg(HV_REG_PDRVCTL, &u8PREDRIDATA);
  if(eErrorState == ERROR)
  {
    printf("Read HV_REG_PDRVCTL FAIL\n");
    return 0;
  }
	/*2. Then set 'PDRVCTL.FRCCFM'*/
  eErrorState = HV_WriteReg(HV_REG_PDRVCTL, (u8PREDRIDATA & (~0x40)) | PDRVCTL_ALL_FRCCFM_FORCE);
  if(eErrorState == ERROR)
  {
    printf("Reset PDRVCTL.FRCCFM bit FAIL\n");
    return ERROR;
  }

  /* Wait for 200Us as manual described */
  Delay_Us(200);

  /* 
   * Init PWM0~5
   *
   * Warnning : 
   *           when PWM is used for PRE-DRIVER, the dead-band of the pair complementary waveform of one PWM channel
   *           must not equal zero, bigger than 600 is recommanded.
   */
  PWM_ComplementaryPairChannelInit(PRE_DRIVER_PWM_U, PRE_DRIVER_PWM_FREQ, PWM_DB_NS);
  PWM_ComplementaryPairChannelInit(PRE_DRIVER_PWM_V, PRE_DRIVER_PWM_FREQ, PWM_DB_NS);
  PWM_ComplementaryPairChannelInit(PRE_DRIVER_PWM_W, PRE_DRIVER_PWM_FREQ, PWM_DB_NS);

  /* 
   * PWM_U set CMPA and loading the same value to PWM_V/PWM_W synchronously, LINK must be set earlier than CAMP value is set,
   * otherwaise, the LINK function will be invalid.
   */
  PWM_LinkCMPA(PRE_DRIVER_PWM_V, PRE_DRIVER_LINK_PWM_U);
  PWM_LinkCMPA(PRE_DRIVER_PWM_W, PRE_DRIVER_LINK_PWM_U);

  /* Set PWM5A output 75% duty waveform */
  u32PWMPeriod = PWMPeriod(PRE_DRIVER_PWM_FREQ);
  PWM_SetCMPA(PRE_DRIVER_PWM_U, u32PWMPeriod / 4);
  
  /* Set PWM of PRE-DRIVER start to count */
  PWM_RunCounter(PRE_DRIVER_PWM_U);
  PWM_RunCounter(PRE_DRIVER_PWM_V);
  PWM_RunCounter(PRE_DRIVER_PWM_W);
  
  GPIO_SetPinChannel(GPIO_36,GPIO36_PWM5A);
  GPIO_SetPinChannel(GPIO_37,GPIO37_PWM5B);

  while(1)
  {

  }
}
