/**************************************************************************//**
 * @file     main.c
 * @brief    Main program body
 * @version  V1.1.0
 * @date     14-Feb-2019
 *
 * @note
 * Copyright (C) 2019 Spintrol Limited Corporation. All rights reserved.
 *
 * @attention
 * THIS SOFTWARE JUST PROVIDES CUSTOMERS WITH CODING INFORMATION REGARDING 
 * THEIR PRODUCTS, WHICH AIMS AT SAVING TIME FOR THEM. SPINTROL SHALL NOT BE
 * LIABLE FOR THE USE OF THE SOFTWARE. SPINTROL DOES NOT GUARANTEE THE 
 * CORRECTNESS OF THIS SOFTWARE AND RESERVES THE RIGHT TO MODIFY THE SOFTWARE 
 * WITHOUT NOTIFICATION.
 *
 ******************************************************************************/

#include "spd1148.h"
#include <stdio.h>

uint8_t                     u8PREDRIID                                    = 0;                 /* PRE-DRIVER mode ID */
uint8_t                     u8PREDRIDATA;                                                       /* Data read from PRE-DRIVER */
ErrorStatus                 eErrorState;
uint32_t                    u32SSPBaundrate                               = 1000000;           /* The baundrate of SSP communicating with PRE-DRIVER mode */
uint32_t                    u32Timeout                                    = 500;               /* The number of times try to get PRE-DRIVER mode ID */
uint8_t                     Count;

/*************************************************************************************************************************
 *
 * @brief      This case, we set over temperature threshold 7 centigrade, if the chip temperatture is over the threshold 
 *             value, corresponding information will be printed.
 *
 *             Key point:
 *						 1. Because of the BUCK current limit of SPD1148 is 200mA, which can cause chip burned out, so over-temperature
 *								event shut down pre-driver function enable is recommended.
 *             2. PRE-DRIVER init include two step,
 *                                             <1> Init the communication circuits(SSP circyits) with MCU, we call this step
 *                                                 'Init PRE-DRIVER module'.
 *                                             <2> Init PRE-DRIVER socket itself, we call this step 'Power up PRE-DRIVER mode'.
 *
 *************************************************************************************************************************/
 
int main(void)
{
  FLASH_WALLOW();
  FLASH_SetTiming(200000000);
  /* Disable flash write access after flash operation had done */
  FLASH_WDIS();

  CLOCK_InitWithRCO(CLOCK_HCLK_200MHZ);

  Delay_Init();

  /*
   * Init the UART
   *
   * 1.Set the GPIO34/35 as UART FUNC
   *
   * 2.Enable the UART CLK
   *
   * 3.Set the rest para
   */
  GPIO_SetPinChannel(GPIO_34,GPIO34_UART_TXD);
  GPIO_SetPinChannel(GPIO_35,GPIO35_UART_RXD);
  CLOCK_EnableModule(UART_MODULE);
  UART_Init(UART,38400);
  
  /* Init PRE-DRIVER module(this step means 'Init PRE-DRIVER module') and get the ID */
  eErrorState = HV_Init(u32SSPBaundrate, &u8PREDRIID, u32Timeout);
  if(eErrorState == ERROR)
  {
    printf("Init PRE-DRIVER mode FAIL\n");
    return 0;
  }
  else
  {
    printf("Init PRE-DRIVER mode SUCCESS[ID:%d]\n",u8PREDRIID);
  }
  
  /* Read out the register value of OTTH */
  eErrorState = HV_ReadReg(HV_REG_OTTH, &u8PREDRIDATA);
  if(eErrorState == ERROR)
  {
    printf("Read OTTH FAIL\n");
    return 0;
  }

  /* 
	 * Set over temperature event0 threshold as 102 centigrade, 
	 * and over temperature event0 threshold as 149 centigrade. 
	 */
  eErrorState = HV_WriteReg(HV_REG_OTTH, ((u8PREDRIDATA & 0x0) & (OTTH_ALL_OT0_102_DEGREE | OTTH_ALL_OT1_149_DEGREE)));
  if(eErrorState == ERROR)
  {
    printf("Set Over temperature value FAIL\n");
    return ERROR;
  }
	
  /* Select Over tmemperature event0 as TZ0 trigger source */
  eErrorState = HV_WriteReg(HV_REG_PDRVTZ0SEL0, PDRVTZ0SEL0_ALL_OT0_ENABLE);
  if(eErrorState == ERROR)
  {
    printf("Set over temperature event0 as TZ0 FAIL\n");
    return 0;
  }
	
  /* Select Over tmemperature event1 as TZ1 trigger source */
  eErrorState = HV_WriteReg(HV_REG_PDRVTZ1SEL0, PDRVTZ1SEL0_ALL_OT1_ENABLE);
  if(eErrorState == ERROR)
  {
    printf("Set over temperature event0 as TZ0 FAIL\n");
    return 0;
  }

  /* Before operate PDRVCTL register, we need to set CTLKEY register to 0x99 */
  eErrorState = HV_WriteReg(HV_REG_CTLKEY, CTLKEY_ALL_KEY_PDRVEN);
  if(eErrorState == ERROR)
  {
    printf("Write CTLKEY register FAIL\n");
    return 0;
  }

  /* 
	 * Power up PRE-DRIVER mode(this step means 'Power up PRE-DRIVER mode') and
	 * enable TZ0 & TZ1 event disable PRE-DRIVER module function.
	*/
  eErrorState = HV_WriteReg(HV_REG_PDRVCTL, PDRVCTL_ALL_EN_ENABLE | PDRVCTL_ALL_TZ0EN_ENABLE | PDRVCTL_ALL_TZ1EN_ENABLE);
  if(eErrorState == ERROR)
  {
    printf("Power up PRE-DRIVER FAIL\n");
    return 0;
  }

  /* Wait for PRE-DRIVER mode become enable */
  while(((u8PREDRIDATA & (0x1 << 6)) == 0) && (Count < 10))
  {
    eErrorState = HV_ReadReg(HV_REG_GLBSTS0, &u8PREDRIDATA);
    if(eErrorState == ERROR)
    {
      printf("PRE-DRIVER enable FAIL[%02x]\n",u8PREDRIDATA);
      return 0;
    }
    if(Count > 10)
    {
      printf("PRE-DRIVER enable timeout\n");
      return 0;
    }
    Count ++;
  }

  /* additional delay after VDDG ready, for final power settling */
  Delay_Us(100);
	
  /* 
   * Clear status register before we start to read the status register, because noise may cause over temperature
   * flag during PRE-DRIVER power up.
   */
  eErrorState = HV_WriteReg(HV_REG_GLBSTSCLR0, GLBSTSCLR0_ALL_OT0_CLEAR);
  if(eErrorState == ERROR)
  {
    printf("Clear status FAIL\n");
    return 0;
  }

  while(1)
  {
    /* Read status register */
    eErrorState = HV_ReadReg(HV_REG_GLBSTS0, &u8PREDRIDATA);
    if(eErrorState == ERROR)
    {
      printf("Read GLBSTS0 register FAIL\n");
      return 0;
    }
    else
    {
      /* Check if over temperature event had happened */
      if((u8PREDRIDATA & (1 << 4)) || (u8PREDRIDATA & (1 << 5)))
      {
        printf("Over temperature had happened\n");
        /* Clear the over temperature flag */
        eErrorState = HV_WriteReg(HV_REG_GLBSTSCLR0, GLBSTSCLR0_ALL_OT0_CLEAR | GLBSTSCLR0_ALL_OT1_CLEAR);
        if(eErrorState == ERROR)
        {
          printf("Clear over temperature FAIL\n");
          return 0;
        }
      }
    }

    Delay_Ms(2000);
  }
}
