/**************************************************************************//**
 * @file     main.c
 * @brief    Main program body
 * @version  V1.1.0
 * @date     14-Feb-2019
 *
 * @note
 * Copyright (C) 2019 Spintrol Limited Corporation. All rights reserved.
 *
 * @attention
 * THIS SOFTWARE JUST PROVIDES CUSTOMERS WITH CODING INFORMATION REGARDING 
 * THEIR PRODUCTS, WHICH AIMS AT SAVING TIME FOR THEM. SPINTROL SHALL NOT BE
 * LIABLE FOR THE USE OF THE SOFTWARE. SPINTROL DOES NOT GUARANTEE THE 
 * CORRECTNESS OF THIS SOFTWARE AND RESERVES THE RIGHT TO MODIFY THE SOFTWARE 
 * WITHOUT NOTIFICATION.
 *
 ******************************************************************************/

#include "spd1148.h"
#include <stdio.h>

/* As descripted below, ADC result convert to voltage can calculate as the follow */
#define                     ValueToVoltage(x)                            ((x * 3657) / 8192) /1000
#define                     SAMPLETIMENS                                 1000 * 1000           /* 1Ms, VDDG convert time no more less than 1Ms */
#define                     CONVTIMENS                                   500


uint8_t                     u8PREDRIID                                    = 0;                 /* PRE-DRIVER mode ID */
uint8_t                     u8PREDRIDATA;                                                       /* Data read from PRE-DRIVER */
ErrorStatus                 eErrorState;
uint32_t                    u32SSPBaundrate                               = 1000000;           /* The baundrate of SSP communicating with PRE-DRIVER mode */
uint32_t                    u32Timeout                                    = 500;               /* The number of times try to get PRE-DRIVER mode ID */
uint8_t                     Count;
int16_t                     i16VSP;


 /*************************************************************************************************************************
 *
 * @brief      Some time, we want to know the exactly voltage value of VDDG/20 internal the chip, this demo code give a method
 *             how to test the VDDG/20 internal value.
 *
 *             The map blow is just a diagram, no the exacly circuit in chip.
 * 
 *                                                              SPD1188 Chip
 *                              -------------------------------------------------------------------
 *                             |                                                                   |
 *                             |      ---------------------                                        |
 *                             |     |            -------  |                    ------------       |
 *                             |     |    ADC    | ATEST |-x-------------------x pre-driver |------x<--------VBAT
 *                             |     |            -------  |                   |  ATEST pin |      |
 *                             |      ---------------------                     ------------       |
 *                             |                                                                   |
 *                              -------------------------------------------------------------------
 *
 *             Key point:
 *                 PRE-DRIVER init include two step
 *                     <1> Init the communication circuits(SSP circyits) with MCU, we call this step 'Init PRE-DRIVER module'.
 *                     <2> Init PRE-DRIVER socket itself, we call this step 'Power up PRE-DRIVER mode'.
 *
 *************************************************************************************************************************/


int main(void)
{
  FLASH_WALLOW();
  FLASH_SetTiming(200000000);
  /* Disable flash write access after flash operation had done */
  FLASH_WDIS();

  CLOCK_InitWithRCO(CLOCK_HCLK_200MHZ);

  Delay_Init();

  /*
   * Init the UART
   *
   * 1.Set the GPIO34/35 as UART FUNC
   *
   * 2.Enable the UART CLK
   *
   * 3.Set the rest para
   */
  GPIO_SetPinChannel(GPIO_34,GPIO34_UART_TXD);
  GPIO_SetPinChannel(GPIO_35,GPIO35_UART_RXD);
  CLOCK_EnableModule(UART_MODULE);
  UART_Init(UART,38400);

  /* Init PRE-DRIVER module(this step means 'Init PRE-DRIVER module') and get the ID */
  eErrorState = HV_Init(u32SSPBaundrate, &u8PREDRIID, u32Timeout);
  if(eErrorState == ERROR)
  {
    printf("Init PRE-DRIVER mode FAIL\n");
    return 0;
  }
  else
  {
    printf("Init PRE-DRIVER mode SUCCESS[ID:%d]\n",u8PREDRIID);
  }

  /* PDRVCTL register can only be writable when CTLKEY=0x99 */
  eErrorState = HV_WriteReg(HV_REG_CTLKEY, CTLKEY_ALL_KEY_PDRVEN);
  if(eErrorState == ERROR)
  {
    printf("Write CTLKEY register FAIL\n");
    return 0;
  }

  /* Power up PRE-DRIVER mode(this step means 'Power up PRE-DRIVER mode') */
  eErrorState = HV_WriteReg(HV_REG_PDRVCTL, PDRVCTL_ALL_EN_ENABLE);
  if(eErrorState == ERROR)
  {
    printf("Power up PRE-DRIVER FAIL\n");
    return 0;
  }

  /* Delay 100us to wait for the VDDG LDO ready */
  Delay_Us(100);

  /* Wait for PRE-DRIVER mode become enable */
  while(((u8PREDRIDATA & (0x1 << 5)) == 0) && (Count < 10))
  {
    eErrorState = HV_ReadReg(HV_REG_GLBSTS0, &u8PREDRIDATA);
    if(eErrorState == ERROR)
    {
      printf("PRE-DRIVER enable FAIL[%02x]\n",u8PREDRIDATA);
      return 0;
    }
    if(Count > 10)
    {
      printf("PRE-DRIVER enable timeout\n");
      return 0;
    }
    Count ++;
  }

  /* additional delay after VDDG ready, for final power settling */
  Delay_Us(100);

  /* 
   * Because high-speed VBAT divider for fast ADC sampler can not be used with the function which
   * VBAT/50 and VDDG/20 monitored by ADC at the same time, so, we need to disable high-speed VBAT divider for fast
   * ADC sampler first.
   */

  /* Read out the old value in SYSCTL register first */
  eErrorState = HV_ReadReg(HV_REG_SYSCTL, &u8PREDRIDATA);
  if(eErrorState == ERROR)
  {
    printf("Read SYSCTL FAIL\n");
    return 0;
  }

  /* Disable high-speed VBAT divider for fast ADC sampler */
  eErrorState = HV_WriteReg(HV_REG_SYSCTL, (u8PREDRIDATA & (~0x80)) | SYSCTL_ALL_VBATDIVEN_DISABLE);
  if(eErrorState == ERROR)
  {
    printf("Disable high-speed VBAT divider for fast ADC sampler FAIL\n");
    return 0;
  }

  /* Read out the old value in VMONCTL register first */
  eErrorState = HV_ReadReg(HV_REG_VMONCTL, &u8PREDRIDATA);
  if(eErrorState == ERROR)
  {
    printf("Read VMONCTL FAIL\n");
    return 0;
  }

  /* Enable voltage supply monitor and select test point as VDDG/20 */
  eErrorState = HV_WriteReg(HV_REG_VMONCTL, (u8PREDRIDATA & (~0x3)) | VMONCTL_ALL_EN_ENABLE | VMONCTL_ALL_TPSEL_VDDG_DIV_20);
  if(eErrorState == ERROR)
  {
    printf("Enable voltage supply monitor and select test point as VDDG/20 FAIL\n");
    return 0;
  }

  /* Connect pre-driver ATEST pin to ADC ATEST pin */
  HV_ConnectAnalogTestToADC();

  /* Init ADC ATEST pin as the positive input of sampler B, the negtive input is the GND */
  ADC_EasyInit1(ADC_SOC_0, ADCx_ATEST, ADCTRIG_Software);

  /* The min convert time of VBAT is 1Ms */
  ADC_SetSampleAndConvertTime(ADC_SOC_0, SAMPLETIMENS, CONVTIMENS);

  while(1)
  {
    /* Use software to trigger ADC SOC0 to work */
    ADC_SoftwareTrigger(ADC_SOC_0);

    /* Wait until ADC conversion finished (Interrupt flag was set) */
    while(!ADC_GetIntFlag(ADC_SOC_0));

    /* Result gotten from 'ADC_GetResult()' can be a negtive value or a positive value */
    i16VSP = ADC_GetResult(ADC_SOC_0);
    printf("ADC SOC0 Voltage = %2fV(Raw data is %d)\n", (double)ValueToVoltage(i16VSP),i16VSP);

    /* Result gotten from 'ADC_GetTrimResult1()' is a positive value */
    i16VSP = ADC_GetTrimResult1(ADC_SOC_0);
    printf("ADC SOC0 Voltage = %2fV(Trim data is %d)\n", (double)ValueToVoltage(i16VSP),i16VSP);

    /* Clear ADC SOC0 INT flag */
    ADC_ClearInt(ADC_SOC_0);

    Delay_Ms(5000);
  }
}
