/******************************************************************************
 * @file     main.c
 * @brief    Main program body
 * @version  V1.8.0
 * @date     31-Dec-2021
 *
 * @note
 * Copyright (C) 2021 Spintrol Limited Corporation. All rights reserved.
 *
 * @attention
 * THIS SOFTWARE JUST PROVIDES CUSTOMERS WITH CODING INFORMATION REGARDING
 * THEIR PRODUCTS, WHICH AIMS AT SAVING TIME FOR THEM. SPINTROL SHALL NOT BE
 * LIABLE FOR THE USE OF THE SOFTWARE. SPINTROL DOES NOT GUARANTEE THE
 * CORRECTNESS OF THIS SOFTWARE AND RESERVES THE RIGHT TO MODIFY THE SOFTWARE
 * WITHOUT NOTIFICATION.
 *
 ******************************************************************************/

#include"spc1168.h"
#include<stdio.h>

/* This macro is used to get the PWM period with a specified frequency */
#define              PWMPeriod(u32PWMFreqHz)             ((CLOCK_GetModuleClock(PWM_MODULE))/u32PWMFreqHz)/2;

#define              PWM_FREQ                            20000                               /* 20kHz PWM */

uint32_t             u32PWMPeriod;


/*

 */
/*************************************************************************************************************************
 *
 * @brief      In this case, 20kHz waveform with 50% duty will be output through PWM1 B channel, bringing out by PWM up-down 
 *             counting mode. we can measure this waveform with GPIO37 connected to a oscilloscope.
 *
 *             GPIO30 set as CBC TZ0 signal and trigger event when it is low, GPIO31/32 set as one-shot TZ1/2 signal and 
 *             trigger event when it is low. As a result, when GPIO30 is low PWM disable the output waveform untill TZ flag
 *             is clear. when GPIO31/32 is low, PWM disable output waveform and enable ouyput waveform immediately at the 
 *             next PWM clock.
 *
 *************************************************************************************************************************/


int main()
{
  FLASH_WALLOW();
  FLASH_SetTiming(200000000);
  /* Disable flash write access after flash operation had done */
  FLASH_WDIS();

  CLOCK_InitWithRCO(CLOCK_HCLK_200MHZ);

  Delay_Init();

  /*
   * Init the UART
   *
   * 1.Set the GPIO34/35 as UART FUNC
   *
   * 2.Enable the UART CLK
   *
   * 3.Set the rest para
   */
  GPIO_SetPinChannel(GPIO_34,GPIO34_UART_TXD);
  GPIO_SetPinChannel(GPIO_35,GPIO35_UART_RXD);
  CLOCK_EnableModule(UART_MODULE);
  UART_Init(UART,38400);

  /* Select GPIO37 as the channel B output of PWM1 */
  GPIO_SetPinChannel(GPIO_37, GPIO37_PWM1B);

  /* Init PWM1 and output 20kHz waveform on channel B */
  PWM_SingleChannelInit(PWM1, PWM_CHB, PWM_FREQ);

  /* Configure PWM counter as up counting mode */
  PWM_SetCounterMode(PWM1, COUNT_DOWN);

  /*
   * The PWM counting mode is set to be up-down by default, as a result, we need 
   * to re-config the AQCTLA register, which is used for controlling the PWM output
   * action when ZRO/CAU/PRD/CAD (for more infomation, please read the PWM application
   * note) event had happened.
   * 
   * Key Point: the PWM freq will be double because of the ZRO/CAU/PRD/CAD event.
   */
   PWM_ActionQualifierCHB(PWM1, AQCTLB_ALL_ZRO_SET_HIGH
                              | AQCTLB_ALL_CAU_DO_NOTHING
                              | AQCTLB_ALL_PRD_DO_NOTHING
                              | AQCTLB_ALL_CAD_SET_LOW);

  /* Set PWM1B output 50% duty waveform */
  u32PWMPeriod = PWMPeriod(PWM_FREQ);
  PWM_SetCMPA(PWM1,u32PWMPeriod / 2);

  /* Start PWM1 */
  PWM_RunCounter(PWM1);

  /*
   * TZ0 event happen when GPIO30 is low, The GPIO need to be configured as input and 
   * select the GPIO as the trip-zone source. 'PWM_SetTZ0FromGPIO()' had achieved this
   * function.
   *
   * 
   * Key point : Beacuse of the GPIO is pulled up internal by MCU. so, TZ triggered by
   *             GPIO low level is recommended.
   */
  PWM_SetTZ0FromGPIO(GPIO_30, GPIO_LEVEL_LOW);

  /* Trigger TZ1 event when GPIO31 is low */
  PWM_SetTZ1FromGPIO(GPIO_31, GPIO_LEVEL_LOW);

  /* Trigger TZ2 event when GPIO32 is low */
  PWM_SetTZ2FromGPIO(GPIO_32, GPIO_LEVEL_LOW);

  /*
   * Set TZ0 as one-shot trip event. The one-shot mode means once the corresponding event
   * has happended, the PWM waveform will stop, and will not start untill the flag is cleared.
   *
   * The key poiny is 'TZEVT_ASYNC_ONLY_FOR_DEBUG' can not be set in practical engineering,
   * it can only be used in DEBUG mode.
   */
  PWM_SetOneShotTripEvent(PWM1, TRIP_EVENT_TZ0, TZSEL_ALL_OSTOUT_ASYNC_OR_LATCH);

  /*
   * Set TZ1 and TZ2 as CBC trip event. The symbol 'CBC' means once the corressponding event
   * has happended, the PWM waveform will stop, but start again at the next PWM CLK period 
   * with do nothing manually.
   *
   * The key point is the same as 'PWM_SetOneShotTripEvent()'
   */
  PWM_SetCBCTripEvent(PWM1,TRIP_EVENT_TZ1|TRIP_EVENT_TZ2, TZSEL_ALL_OSTOUT_ASYNC_OR_LATCH);

  /* 
   * Set PWM1 output as tristate upon one-shot and CBC trip event,need to explicitly specify
   * actions for all 6 trip scenarios 
   */
  PWM_SetCHAOutputWhenTrip(PWM1, TZU_TRIP_AS_TRI_STATE |
                                 TZD_TRIP_AS_TRI_STATE |
                                 DCEVT0U_TRIP_DO_NOTHING |
                                 DCEVT0D_TRIP_DO_NOTHING |
                                 DCEVT1U_TRIP_DO_NOTHING |
                                 DCEVT1D_TRIP_DO_NOTHING);

  PWM_SetCHBOutputWhenTrip(PWM1, TZU_TRIP_AS_TRI_STATE |
                                 TZD_TRIP_AS_TRI_STATE |
                                 DCEVT0U_TRIP_DO_NOTHING |
                                 DCEVT0D_TRIP_DO_NOTHING |
                                 DCEVT1U_TRIP_DO_NOTHING |
                                 DCEVT1D_TRIP_DO_NOTHING);

  while(1)
  {

  }
}
